# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from torch.utils.data.dataset import ConcatDataset


def _maker(protocol):
    if protocol == "default":
        from ..indian_RS import default as indian
        from ..montgomery_RS import default as mc
        from ..shenzhen_RS import default as ch
        from ..tbx11k_simplified_RS import default as tbx11k
    elif protocol == "rgb":
        from ..indian_RS import rgb as indian
        from ..montgomery_RS import rgb as mc
        from ..shenzhen_RS import rgb as ch
        from ..tbx11k_simplified_RS import rgb as tbx11k
    elif protocol == "fold_0":
        from ..indian_RS import fold_0 as indian
        from ..montgomery_RS import fold_0 as mc
        from ..shenzhen_RS import fold_0 as ch
        from ..tbx11k_simplified_RS import fold_0 as tbx11k
    elif protocol == "fold_1":
        from ..indian_RS import fold_1 as indian
        from ..montgomery_RS import fold_1 as mc
        from ..shenzhen_RS import fold_1 as ch
        from ..tbx11k_simplified_RS import fold_1 as tbx11k
    elif protocol == "fold_2":
        from ..indian_RS import fold_2 as indian
        from ..montgomery_RS import fold_2 as mc
        from ..shenzhen_RS import fold_2 as ch
        from ..tbx11k_simplified_RS import fold_2 as tbx11k
    elif protocol == "fold_3":
        from ..indian_RS import fold_3 as indian
        from ..montgomery_RS import fold_3 as mc
        from ..shenzhen_RS import fold_3 as ch
        from ..tbx11k_simplified_RS import fold_3 as tbx11k
    elif protocol == "fold_4":
        from ..indian_RS import fold_4 as indian
        from ..montgomery_RS import fold_4 as mc
        from ..shenzhen_RS import fold_4 as ch
        from ..tbx11k_simplified_RS import fold_4 as tbx11k
    elif protocol == "fold_5":
        from ..indian_RS import fold_5 as indian
        from ..montgomery_RS import fold_5 as mc
        from ..shenzhen_RS import fold_5 as ch
        from ..tbx11k_simplified_RS import fold_5 as tbx11k
    elif protocol == "fold_6":
        from ..indian_RS import fold_6 as indian
        from ..montgomery_RS import fold_6 as mc
        from ..shenzhen_RS import fold_6 as ch
        from ..tbx11k_simplified_RS import fold_6 as tbx11k
    elif protocol == "fold_7":
        from ..indian_RS import fold_7 as indian
        from ..montgomery_RS import fold_7 as mc
        from ..shenzhen_RS import fold_7 as ch
        from ..tbx11k_simplified_RS import fold_7 as tbx11k
    elif protocol == "fold_8":
        from ..indian_RS import fold_8 as indian
        from ..montgomery_RS import fold_8 as mc
        from ..shenzhen_RS import fold_8 as ch
        from ..tbx11k_simplified_RS import fold_8 as tbx11k
    elif protocol == "fold_9":
        from ..indian_RS import fold_9 as indian
        from ..montgomery_RS import fold_9 as mc
        from ..shenzhen_RS import fold_9 as ch
        from ..tbx11k_simplified_RS import fold_9 as tbx11k
    elif protocol == "fold_0_rgb":
        from ..indian_RS import fold_0_rgb as indian
        from ..montgomery_RS import fold_0_rgb as mc
        from ..shenzhen_RS import fold_0_rgb as ch
        from ..tbx11k_simplified_RS import fold_0_rgb as tbx11k
    elif protocol == "fold_1_rgb":
        from ..indian_RS import fold_1_rgb as indian
        from ..montgomery_RS import fold_1_rgb as mc
        from ..shenzhen_RS import fold_1_rgb as ch
        from ..tbx11k_simplified_RS import fold_1_rgb as tbx11k
    elif protocol == "fold_2_rgb":
        from ..indian_RS import fold_2_rgb as indian
        from ..montgomery_RS import fold_2_rgb as mc
        from ..shenzhen_RS import fold_2_rgb as ch
        from ..tbx11k_simplified_RS import fold_2_rgb as tbx11k
    elif protocol == "fold_3_rgb":
        from ..indian_RS import fold_3_rgb as indian
        from ..montgomery_RS import fold_3_rgb as mc
        from ..shenzhen_RS import fold_3_rgb as ch
        from ..tbx11k_simplified_RS import fold_3_rgb as tbx11k
    elif protocol == "fold_4_rgb":
        from ..indian_RS import fold_4_rgb as indian
        from ..montgomery_RS import fold_4_rgb as mc
        from ..shenzhen_RS import fold_4_rgb as ch
        from ..tbx11k_simplified_RS import fold_4_rgb as tbx11k
    elif protocol == "fold_5_rgb":
        from ..indian_RS import fold_5_rgb as indian
        from ..montgomery_RS import fold_5_rgb as mc
        from ..shenzhen_RS import fold_5_rgb as ch
        from ..tbx11k_simplified_RS import fold_5_rgb as tbx11k
    elif protocol == "fold_6_rgb":
        from ..indian_RS import fold_6_rgb as indian
        from ..montgomery_RS import fold_6_rgb as mc
        from ..shenzhen_RS import fold_6_rgb as ch
        from ..tbx11k_simplified_RS import fold_6_rgb as tbx11k
    elif protocol == "fold_7_rgb":
        from ..indian_RS import fold_7_rgb as indian
        from ..montgomery_RS import fold_7_rgb as mc
        from ..shenzhen_RS import fold_7_rgb as ch
        from ..tbx11k_simplified_RS import fold_7_rgb as tbx11k
    elif protocol == "fold_8_rgb":
        from ..indian_RS import fold_8_rgb as indian
        from ..montgomery_RS import fold_8_rgb as mc
        from ..shenzhen_RS import fold_8_rgb as ch
        from ..tbx11k_simplified_RS import fold_8_rgb as tbx11k
    elif protocol == "fold_9_rgb":
        from ..indian_RS import fold_9_rgb as indian
        from ..montgomery_RS import fold_9_rgb as mc
        from ..shenzhen_RS import fold_9_rgb as ch
        from ..tbx11k_simplified_RS import fold_9_rgb as tbx11k

    mc = mc.dataset
    ch = ch.dataset
    indian = indian.dataset
    tbx11k = tbx11k.dataset

    dataset = {}
    dataset["__train__"] = ConcatDataset(
        [
            mc["__train__"],
            ch["__train__"],
            indian["__train__"],
            tbx11k["__train__"],
        ]
    )
    dataset["train"] = ConcatDataset(
        [mc["train"], ch["train"], indian["train"], tbx11k["train"]]
    )
    dataset["__valid__"] = ConcatDataset(
        [
            mc["__valid__"],
            ch["__valid__"],
            indian["__valid__"],
            tbx11k["__valid__"],
        ]
    )
    dataset["validation"] = ConcatDataset(
        [
            mc["validation"],
            ch["validation"],
            indian["validation"],
            tbx11k["validation"],
        ]
    )
    dataset["test"] = ConcatDataset(
        [mc["test"], ch["test"], indian["test"], tbx11k["test"]]
    )

    return dataset
