# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""CNN for radiological findings detection.

A Densenet121 model for radiological extraction
"""

from torch import empty
from torch.nn import BCEWithLogitsLoss

from ...models.densenet_rs import DensenetRS

# Import the default protocol if none is available
if "dataset" not in locals():
    from ..datasets.nih_cxr14_re import default

    dataset = default.dataset

# config
optimizer_configs = {"lr": 1e-4}

# optimizer
optimizer = "Adam"

# criterion
criterion = BCEWithLogitsLoss(pos_weight=empty(1))
criterion_valid = BCEWithLogitsLoss(pos_weight=empty(1))

# model
model = DensenetRS(criterion, criterion_valid, optimizer, optimizer_configs)
